--[[ MockObject ]]

local MockMeta = {}

local assertEquals = function(message, a, b)
	local helpers = {};
	helpers.compareTables = function(x, y, tableName)
		local fieldHeader = "";
		if tableName ~= nil then fieldHeader = tableName .. "." end;
			
		for k, v in pairs(x) do
			local result, expected, actual, field = helpers.areEqual(v, y[k], fieldHeader .. tostring(k));
			if result == false then return false, expected, actual, field end;
		end		
		return true;
	end;
	helpers.compare = function(x, y, name)
		if type(x) ~= type(y) then return false, x, y, name end;
		if type(x) == "table" then
			local result, expected, actual, field = helpers.compareTables(x, y, name);
			if result == false then return false, expected, actual, field end;
				
			result, actual, expected, field = helpers.compareTables(y, x, name);
			if result == false then return false, expected, actual, field end;
				
			return true;
		end;
		if x == y then
			return true;
		else
			return false, x, y, name;
		end;
	end;
	helpers.areEqual = function(x, y, field)
		return helpers.compare(x, y, field)
	end;
		
	if b == nil then
		b = a;
		a = message;
		message = "";
	else
		message = message .. ": ";
	end
		
	local result, expected, actual, field = helpers.areEqual(a, b);
		
	message = message .. "expected:<" .. tostring(expected) .. "> but was:<" .. tostring(actual) .. ">";
	if field ~= nil then
		message = message .. " for field <" .. tostring(field) .. ">";
	end
		
	if result == false then
		error(message, 3)
	end
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

function MockMeta.__index(object, methodName)
	local mock = object.mockData;

	if((mock.mode ~= "record") and (mock.mode ~= "replay")) then 
		error("Unknown mock mode: " .. mock.mode);
	end;
	
	if(methodName == "reset") then
		return function()
			mock.mode = "record";
			mock.callIndex = 0;
			mock.callList = {};
		end
	end;
	
	if(methodName == "replay") then
		return function()
			mock.mode = "replay";
			mock.callIndex = 0;
		end
	end;
	
	if(methodName == "verify") then
		if(mock.mode == "record") then 
			error("Can not verify while recording mock behavior");
		end
		
		return function()
			if(mock.callList[mock.callIndex] ~= nil) then 
				error("Uncalled method during replay: " .. mock.callList[mock.callIndex].methodName , 4);
			end
		end
	end;
	
	if(mock.mode == "record") then
		return function(self, ...)
			local m = {};
			m.methodName = methodName;
			m.methodArgs = {...} or {};
			
			mock.callList[mock.callIndex] = m;
			mock.callIndex = mock.callIndex + 1;
			
			return m;
		end;
	else
		return function(self, ...)
			local m = {};
			m.methodName = methodName;
			m.methodArgs = {...} or {};
			
			assertEquals("Unexpected method call", mock.callList[mock.callIndex].methodName, m.methodName);
			assertEquals("Unexpected method arguments", mock.callList[mock.callIndex].methodArgs, m.methodArgs);
			
			m = mock.callList[mock.callIndex];
			mock.callIndex = mock.callIndex + 1;
			
			if(m.returns) then return unpack(m.returns) end;
		end;
	end;
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

function WoWUnit_CreateMock()
	local rVal = {
		mockData = {
			mode = "record";
			callIndex = 0;
			callList = {};
		}
	};
	setmetatable(rVal, MockMeta);
	return rVal;
end

local
function executeOnAll(method, ...)
	if not ... then return end;
	for _, o in ipairs({...}) do
		o[method]();
	end
end

function WoWUnit_ReplayMocks(...)
	executeOnAll("replay", ...);
end

function WoWUnit_VerifyMocks(...)
	executeOnAll("verify", ...);
end

function WoWUnit_ResetMocks(...)
	executeOnAll("reset", ...);
end

-------------------------------------------------------------------------------
-------------------------------------------------------------------------------
-------------------------------------------------------------------------------

-- Mock unit tests are in WoWUnit.lua
